function decomposition = simply(data,W, NoComp)

% Input
% data:     your data as a matrix (objects x variables)
% W:        W reference to be generated with CalcW.m
% NoComp:   nmber of components to be fitted

% Copyright 2013 Biosystems Data Analysis Group; Universiteit van Amsterdam
%
% This is a license to use and modify SOFTWARE & DATA produced by:
% THE BIOSYSTEMS DATA ANALYSIS GROUP OF THE UNIVERSITEIT VAN AMSTERDAM

% If you use, modify or redistribute the SOFTWARE & DATA and/or your source modifications, you agree: 
% i)	to use the SOFTWARE & DATA and/or your source modifications solely as part of your research and not in any commercial product; 
%	
% ii)	that the SOFTWARE  & DATA and/or your source modifications will not be distributed for profit; 

% iii)	all copyright notices and this license note with the SOFTWARE & DATA are retained any  redistribution of the SOFTWARE & DATA, or any portion thereof; 
%
% iv)	to indemnify, hold harmless, and defend the "Biosystems Data Analysis Group of the Universiteit van Amsterdam" from and against any claims or lawsuits that arise or result from the use of the SOFTWARE & DATA or your source modifications. 
%
% v)	to properly reference the SOFTWARE & DATA when used in your reseach in any publication that may result from that research.
%	
% Reserved Rights. The "Biosystems Data Analysis Group of the Universiteit
% van Amsterdam" retain title and all ownership rights to the SOFTWARE & DATA.


% Matlab (R)  Genetic Algorithm and Direct search Toolboxes are required.

% Saccenti E, Westerhuis JA, Smilde AK, van der Werf MJ, Hageman JA, Hendriks MM. 
% Simplivariate models: uncovering the underlying biology in functional genomics data.
% PLoS One. 2011;6(6):e20747. doi: 10.1371/journal.pone.0020747. Epub 2011 Jun 16.

% Data needs to be provided in a matrix form, rows are the observations
% columns represents measured entities
% W need to be calculated with the CalcW function
% In the code diverse GA parameters can still be optimised, e.g. the population size 
% defined in the function myInit


settings.W = W;
settings.maxSC = NoComp;  % number of simplivariate components
settings.maxperm = 0;

% setting parameters for the GA
settings.repeats = [5 5];
settings.seeds = [10000*randn(1,1) 10000*randn(1,1) 10000*randn(1,1) 10000*randn(1,1) 10000*randn(1,1)]; 
settings.mutationrate = 0.05;       % in @myMutation default = 0.05
settings.numberofgenerations = 750; % Generations default = 100;
settings.numberofstrings = 100;		% PopulationSize default = 20;
settings.crossoverrate = 0.8;  		% CrossoverFraction default = 0.8;
settings.noplot = 1;  
  

% data autoscaling
Xorig = zscore(data);
X = Xorig;
XW = Xorig;

for L=1:settings.maxSC

    % find layer
    fprintf('Searching for simplivariate component %i of %i...',L, settings.maxSC)
    [SimpComp]=find_SC(X, settings, 1);

    % accept SC, store it
    decomposition{L}=SimpComp;

    % backfit all previously found layers and last layer
    
    disp('Entering backfitting procedure')
    
    [X, decomposition]=backfit(Xorig, decomposition, settings);
    
    disp('Backfitting procedure completed')
    fprintf('Search of simplivariate component %i of %i done!\n',L, settings.maxSC)
    disp(' ');

end

%lS(decomposition, labelsy, xlsfile);




% GA FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function find_layer
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function  [SC] = find_SC(X, settings, num)

W = settings.W;
% definition of fitness function
FitnessFcn = @(POP) multiplicative_model_evalfunc(POP, X, settings);

best=[];
fval=[];

% definition of parameters for ga optimization
for repeat=1:settings.repeats(num)
    rand('state', settings.seeds(repeat));
    randn('state', settings.seeds(repeat)+100);

    % initialise GA
    limits(1,:)=repmat(1, 1, sum(size(X,2)));
    limits(2,:)=repmat(2, 1, size(X,2));

    problem.fitnessfcn=FitnessFcn;
    problem.nvars=sum(size(X,2));
    problem.options = gaoptimset('PopInitRange', limits,...
        'CreationFcn', @myInit,...
        'MutationFcn', {@myMutation, settings.mutationrate},...
        'PlotFcns', {@gaplotbestf, @gaplotbestindiv},...
        'Generations', settings.numberofgenerations,...
        'PopulationSize', settings.numberofstrings,...
        'StallGenLimit', floor(0.20*settings.numberofgenerations)+1,...
        'StallTimeLimit', 600,...
        'CrossoverFcn', @myCrossover,...
        'EliteCount', 1,...
        'CrossoverFraction', settings.crossoverrate,...
        'SelectionFcn', @selectionstochunif,...
        'TolFun', 0.0001,...
        'Vectorized','on');

    if (settings.noplot==1)
        problem.options.PlotFcns={[]};
    end

    % perform GA, for each repeat the best solution is saved together with corresponding value
    % of the fitness function
    [best(repeat, :), fval(repeat)]=ga(problem);
end


% find minimum of the repeats
[I J]=min(fval);
best_string=best(J, :);
SC.rho=ones(size(X,1), 1);      % set to ones for one mode clustering only
SC.tau=best_string-1;

% find results for this solution (i.e. parameters)
Xsubset = X(SC.rho==1,SC.tau==1);
SC.alpha = mean(Xsubset);
[uX,sX,vX] = svds((Xsubset - repmat(SC.alpha,size(Xsubset,1),1)),1);
SC.lambda = uX*sX;
SC.kappa  = vX;
% fitted model
SC.multX=SC.lambda*SC.kappa' + repmat(SC.alpha,size(Xsubset,1),1);
SC.SSE = sum(sum((Xsubset - SC.multX).^2));
SC.SST = sum(sum(mncntr(Xsubset).^2));
SC.LSS = sum(sum(SC.multX.^2));
[nXsub, mXsub] = size(Xsubset);
SC.selsize = mXsub;
SC.nXSub =  nXsub;
SC.mXSub =  mXsub;


SC.respons= -(((SC.SST - SC.SSE) ./ ((nXsub-1)*mXsub)) - W(SC.selsize));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  backfit procedure to obtain better estimates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [X, decomp]=backfit(Xorig, decomp, settings)

W = settings.W;


% get residuals
if (size(decomp,2)>1)
    

    for bfl=1:size(decomp,2);
        others=1:size(decomp,2);
        others=others(others~=bfl);
        
        X = Xorig;
        for j=1:size(others,2)
            X(decomp{others(j)}.rho==1, decomp{others(j)}.tau==1) = X(decomp{others(j)}.rho==1, decomp{others(j)}.tau==1) - decomp{others(j)}.multX;
        end
        

        % update SC effect
        Xsubset=X(decomp{bfl}.rho==1,decomp{bfl}.tau==1);
        [uX,sX,vX] = svds((Xsubset - repmat(decomp{bfl}.alpha,size(Xsubset,1),1)),1);
        decomp{bfl}.lambda = uX*sX;
        decomp{bfl}.kappa  = vX;
        decomp{bfl}.multX=decomp{bfl}.lambda*decomp{bfl}.kappa' + repmat(decomp{bfl}.alpha,size(Xsubset,1),1);
        decomp{bfl}.SSE = sum(sum((Xsubset - decomp{bfl}.multX).^2));
        decomp{bfl}.SST = sum(sum(mncntr(Xsubset).^2));
        decomp{bfl}.LSS = sum(sum(decomp{bfl}.multX).^2);
        [nXsub, mXsub] = size(Xsubset);
        



        decomp{bfl}.respons= -((decomp{bfl}.SST - decomp{bfl}.SSE)./ ((nXsub-1)*mXsub) - W(mXsub));
    end
else
    %fprintf('No backfitting possible for only one simplivariate component\n');
end

X = Xorig;
for bfl = 1:size(decomp,2);
    decomp{bfl}.Xbefore = X;
    X(decomp{bfl}.rho==1,decomp{bfl}.tau==1) = X(decomp{bfl}.rho==1,decomp{bfl}.tau==1) - decomp{bfl}.multX;
    decomp{bfl}.Xafter = X;
end


%--------------------------------------------------------------------------
% GA evaluation function for multiplicative models (through SVD decomposition)
%--------------------------------------------------------------------------
function [respons] = multiplicative_model_evalfunc(pop, Z, settings)

W = settings.W;


for go=1:size(pop,1)
    b=pop(go, :)-1;                     % variables clustering
    a=ones(size(Z,1), 1);               % set to ones for one mode clustering only
    Zsubset = Z(a==1,b==1);

    if length(find(b==1)) <= 2;
        SSE = 0;
        respons(go) = Inf;
    else
        alpha = mean(Zsubset);
        [uZ,sZ,vZ] = svds((Zsubset - repmat(alpha,size(Zsubset,1),1)),1);
        lambda = uZ*sZ;
        kappa  = vZ;
        multZ=lambda*kappa' + repmat(alpha,size(Zsubset,1),1);
        SSE = sum(sum((Zsubset - multZ).^2));
        SST = sum(sum(mncntr(Zsubset).^2));

        [nZsub, mZsub] = size(Zsubset);

        respons(go) =   -( (SST - SSE) ./ ( (nZsub-1)*mZsub)  - W(mZsub));
        
       
    end
end



% population is created 
function Population = myInit(GenomeLength, FitnessFcn, options)

Maxx = 15;
Minn = 5;

totalpopulation = sum(options.PopulationSize);
width = round(totalpopulation/(Maxx-Minn));

POP = [];

for num_ones = Minn : Maxx;

    ZER = zeros(width,GenomeLength);
    for i = 1:size(ZER,1);

        index = randsample(1:GenomeLength,num_ones);
        ZER(i,index) = 1;
    end

    POP = [POP;ZER];
    clear ZER
end

Population = POP+1;
clear POP


%--------------------------------------------------------------------------
function mutationChildren = myMutation(parents,options,GenomeLength,FitnessFcn,state,thisScore,thisPopulation,mutationRate)

if (nargin < 8)
    mutationRate = 0.05;
end

mutationChildren = zeros(length(parents),GenomeLength);
for i=1:length(parents)
    child = thisPopulation(parents(i),:);
    mutationPoints = find(rand(1,length(child)) < mutationRate);
    range = options.PopInitRange;

    if numel(mutationPoints~=0)
        for h=1:size(mutationPoints, 2)
            where=mutationPoints(h);
            child(where) = random_number(range(1,where), range(2,where), 1, 1);
        end
    end
    mutationChildren(i,:) = child;
end

%--------------------------------------------------------------------------

function xoverKids = myCrossover(parents,options,GenomeLength,FitnessFcn,unused,thisPopulation)

% how many children to produce?
nKids = length(parents)/2;

% allocate space for the kids
xoverKids = zeros(nKids,GenomeLength);

% to move through the parents twice as fast as thekids are
% being produced, a separate index for the parents is needed
index = 1;

for i=1:nKids
    % get parents
    parent1 = thisPopulation(parents(index),:);
    index = index + 1;
    parent2 = thisPopulation(parents(index),:);
    index = index + 1;

    % choose two (nonequal) crossover points
    sz  = length(parent1) - 1;
    xOverPoint1 = ceil(sz * rand);
    xOverPoint2 = ceil(sz * rand);
    while(xOverPoint2 == xOverPoint1)
        xOverPoint2 = ceil(sz * rand);
    end

    % deal with the case where the splice wraps around the ends
    if(xOverPoint1 < xOverPoint2)
        left = xOverPoint1;
        right = xOverPoint2;
    else
        left = xOverPoint2;
        right = xOverPoint1;
        swap = parent1;
        parent1 = parent2;
        parent2 = swap;
    end

    % make one child
    xoverKids(i,:) = [ parent1(1:left),  parent2(( left + 1 ):  right ), parent1(( right + 1):  end )   ];
end

%-------------------------------------------------------------
function C = mncntr(X)

     C = X - repmat(mean(X),size(X,1),1);

return

%------------------------------------------------------------

function numb=random_number(lowerb, upperb, nr, integer)

if nargin == 2;
    nr=1;
end

if nargin < 2;
    error('Provide lower and upper bounds. Syntax: random_number(lowerb, upperb, nr)');
end

if nargin == 4;
    numb = (rand(1, nr) * (upperb-lowerb+1)) + lowerb;
    numb = floor(numb);
else
    numb = (rand(1, nr) * (upperb-lowerb)) + lowerb;
end

return


